/*
 * Created on 20.2.2005
 *
 */
package net.sf.tomp.el.author;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Collection;
import java.util.TreeMap;
import java.util.Iterator;
import java.util.SortedMap;
import java.util.Properties;


/**
 * @author <a href="mailto:tomp@fi.muni.cz">RNDr. Tomáš Pitner, Ph.D.</a>
 *
 *
 */
public class CourseTree {
    
    private SortedMap<String, Course> courses;
    
    private File root;
    
    private Properties buildProperties;
    
    public static final String buildPropertiesFileName = "build.properties";
    
    private String nameInTree;
    
    /**
     * Creates a new instance of CourseTree ans sets its root file.
     * Creates a new instance of its TreeMap courses.
     * @param root Root file
     */
    public CourseTree(File root) {
        super();
        this.root = root;
        courses = new TreeMap<String, Course>();
        readBuildProperties();
        nameInTree = "Courses (" + getDirectory() + ")";
    }
    
    /**
     * Returns a collection of Courses' values.
     * @return Collection of Courses' values.
     */
    public Collection<Course> getCourses() {
        return courses.values();
    }
    
    /**
     * Returns the name in the GUI tree.
     * @return name in the GUI tree
     */
    public String getNameInTree() {
        return nameInTree;
    }
    
    /**
     * Sets the name in the GUI tree.
     * @param n name in the GUI tree
     */
    public void setNameInTree(String n) {
        nameInTree = n;
    }
    
    /**
     * Adds a Course to a CourseTree.
     * @param c Course to add.
     */
    public void addCourse(Course c) {
        courses.put(c.getId(), c);
    }
    
    /**
     * Created a new Course, makes its directories (also for its lectures),
     * its main-info.xml file and puts the Course into the CourseTree.
     * @param id Course's ID.
     * @return Created Course
     * @exception when
     */
    public Course newCourse(String id) throws IOException {
        Course c = new Course(this, id);
        c.getDirectory().mkdir();
        
        // creating subdirectories for new course...
        c.getLectures().getDirectory().mkdirs();
        c.getTasks().getDirectory().mkdirs();
        c.getPractices().getDirectory().mkdirs();
        c.getProjects().getDirectory().mkdirs();
        
        System.out.println(c.getTasks().getDirectory().getAbsolutePath());
        c.getInfoFile().createNewFile();
        c.getLectures().getInfoFile().createNewFile();
        c.getTasks().getInfoFile().createNewFile();
        c.getPractices().getInfoFile().createNewFile();
        c.getProjects().getInfoFile().createNewFile();
        
        courses.put(c.getId(), c);
        return c;
    }
    
    public void removeCourse(Course c) {
        deleteDir(c.getDirectory());
        courses.remove(c.getId());
    }
    
    /**
     * Deletes a non-empty directory.
     * @param dir directory to delete
     * @return True, if directory is successfully deleted.
     */
    public static boolean deleteDir(File dir) {
        if (dir.isDirectory()) {
            String[] children = dir.list();
            for (int i=0; i<children.length; i++) {
                boolean success = deleteDir(new File(dir, children[i]));
                if (!success) {
                    return false;
                }
            }
        }
        // The directory is now empty so delete it
        return dir.delete();
    }
    
    /**
     * Returns Course from courses.
     * @param id Course's ID.
     * @return Course from courses.
     */
    public Course getCourse(String id) {
        if (id == null)
            return null;
        return courses.get(id);
    }
    
    /**
     * Returns first course in courses sorted map.
     * @return First course from courses
     */
    public Course getFirstCourse() {
        return courses.get(courses.firstKey());
    }
    
    /**
     * Reads build properties from a file.
     */
    public void readBuildProperties() {
        buildProperties = new Properties();
        try {
            buildProperties.load(new FileInputStream(new File(root,
                    buildPropertiesFileName)));
        } catch (IOException ioe) {
            throw new RuntimeException(ioe);
        }
    }
    
    /**
     * Writes build properties to a file.
     */
    public void writeBuildProperties() {
        try {
            buildProperties.store(new FileOutputStream(new File(root,
                    buildPropertiesFileName)),
                    "Course Author Ant build.properties");
        } catch (IOException ioe) {
            throw new RuntimeException(ioe);
        }
    }
    
    /**
     * Returns current term from properties.
     * @return String containing current term.
     */
    public String getCurrentTerm() {
        return buildProperties.getProperty("el.term");
    }
    
    /**
     * Returns current course from properties.
     * @return Current course.
     */
    public Course getCurrentCourse() {
        return getCourse(buildProperties.getProperty("el.subject"));
    }
    
    /**
     * Returns current lecture from properties.
     * @return Current lecture.
     */
    public Lecture getCurrentLecture() {
        Course cc = getCurrentCourse();
        if (cc == null) {
            return null;
        } else {
            return cc.getLectures().getLecture(
                    buildProperties.getProperty("el.lecture"));
        }
    }
    
    /**
     * Returns current parameters from properties.
     * @return Current parameteres.
     */
    public String getCurrentParameters() {
        return buildProperties.getProperty("el.parameters");
    }
    /**
     * Returns principal document.
     * Not implemented yet!
     * @return Principal document.
     */
    public String getPrincipalDoc() {
        return null;
        //		return buildProperties.getProperty("el.principal.doc");
    }
    
    /**
     * Returns the root file.
     * @return Root file.
     */
    public File getDirectory() {
        return root;
    }
    
    /**
     * Creates string of CourseTree's courses.
     * @return String containing CourseTree's courses..
     */
    public String toString() {
        StringBuffer sb = new StringBuffer();
        for (Iterator<Course> i = getCourses().iterator(); i.hasNext();) {
            sb.append(i.next());
        }
        return "courses:\n" + sb;
    }
}
