/*
 * Created on 20.2.2005
 *
 */
package net.sf.tomp.el.author.io;

import java.io.File;
import java.io.FileFilter;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.util.Iterator;

import net.sf.tomp.el.author.Project;
import net.sf.tomp.el.author.Projects;
import net.sf.tomp.el.author.Practice;
import net.sf.tomp.el.author.Practices;
import net.sf.tomp.el.author.Task;
import net.sf.tomp.el.author.Tasks;
import net.sf.tomp.el.author.Course;
import net.sf.tomp.el.author.CourseTree;
import net.sf.tomp.el.author.Lecture;
import net.sf.tomp.el.author.Lectures;
import net.sf.tomp.el.author.description.Description;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentFactory;
import org.dom4j.Element;
import org.dom4j.Namespace;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;

/**
 * @author <a href="mailto:tomp@fi.muni.cz">RNDr. Tomáš Pitner, Ph.D.</a>
 *
 */
public class AuthorIO {
    
    private static final DocumentFactory factory = DocumentFactory
            .getInstance();
    
    /**
     * Creates a new instance of CourseTree and loads the courses
     * of the CourseTree.
     * @param id CourseTree ID
     * @return Loaded CourseTree
     * @throws MalformedURLException
     * @throws DocumentException
     */
    public CourseTree loadCourseTree(String id) throws MalformedURLException,
            DocumentException {
        CourseTree tree = new CourseTree(new File(id));
        loadCourses(tree);
        return tree;
    }
    
    /**
     * Saves the CourseTree
     * @param tree The CourseTree to be saved.
     * @throws MalformedURLException
     * @throws DocumentException
     */
    public void saveCourseTree(CourseTree tree) throws MalformedURLException,
            DocumentException {
        tree.writeBuildProperties();
    }
    
    /**
     * Loads all courses in the tree.
     * @param tree Courses from this tree will be loaded.
     * @throws MalformedURLException
     * @throws DocumentException
     */
    public void loadCourses(CourseTree tree) throws MalformedURLException,
            DocumentException {
        String[] courseIds = listCourseIds(tree);
        for (int i = 0; i < courseIds.length; i++) {
            loadCourse(tree, courseIds[i]);
        }
    }
    
    /**
     * Saves all courses in the tree.
     * @param tree Courses from this tree will be saved
     * @throws UnsupportedEncodingException
     * @throws DocumentException
     * @throws IOException
     */
    public void saveCourses(CourseTree tree) throws UnsupportedEncodingException, DocumentException, IOException {
        for (Iterator<Course> i = tree.getCourses().iterator(); i.hasNext();) {
            saveCourse(i.next());
        }
    }
    
    /**
     * Creates a new instance of Course, adds it to the CourseTree
     * loads the lectures of the course.
     * @param tree Tree where the new Course will be loaded.
     * @param id Course ID
     * @return Loaded Course
     * @throws MalformedURLException
     * @throws DocumentException
     */
    public Course loadCourse(CourseTree tree, String id)
    throws MalformedURLException, DocumentException {
        Course course = new Course(tree, id);
        
        SAXReader xmlReader = new SAXReader();
        Document doc = xmlReader.read(course.getInfoFile());
        Description desc = course.getDescription();
        desc.setTitle(doc.getRootElement().elementText("title"));
        desc.setBody(doc.getRootElement().elementText("body"));
        // description changed, updating...
        course.updateNameInTree();
        tree.addCourse(course);    
        
//        Course course = new Course(tree, id);
//        tree.addCourse(course);
        loadLectures(course);
        loadTasks(course);
        loadPractices(course);
        loadProjects(course);
        
        loadLecturesInfo(course);
        loadTasksInfo(course);
        loadPracticesInfo(course);
        loadProjectsInfo(course);
        return course;
    }
    
    /**
     * Builds up a new XML document for Course using DOM4J factory
     * and saves it. Also saves info-files for course's lectures, tasks,
     * practices and projects.
     * @param course This course will be saved.
     * @throws UnsupportedEncodingException
     * @throws DocumentException
     * @throws IOException
     */
    public void saveCourse(Course course) throws UnsupportedEncodingException, DocumentException, IOException {
        Document doc = factory.createDocument();
        Element root = factory.createElement("section");
        root.addAttribute("id", course.getId());
        doc.add(root);
        Element title = factory.createElement("title");
        title.setText(course.getDescription().getTitle());
        Element body = factory.createElement("body");
        body.setText(course.getDescription().getBody());
        root.add(title);
        root.add(body);
        write(doc, course.getInfoFile());
        
        // creating basic xml-files...
        saveLecturesInfo(course.getLectures());
        saveLecturesInclude(course.getLectures());
        saveTasksInfo(course.getTasks());
        saveTasksInclude(course.getTasks());
        savePracticesInfo(course.getPractices());
        savePracticesInclude(course.getPractices());
        saveProjectsInfo(course.getProjects());
        saveProjectsInclude(course.getProjects());
    }
    
    /**
     * Loads lectures for a course.
     * @param course Lectures from this course will be loaded.
     * @throws MalformedURLException
     * @throws DocumentException
     */
    public void loadLectures(Course course) throws MalformedURLException,
            DocumentException {
        String[] lectureIds = listLectureIds(course.getLectures());
        for (int i = 0; i < lectureIds.length; i++) {
            loadLecture(course, lectureIds[i]);
        }
    }
    
    /**
     * Saves all lectures of the course.
     * @param course lectures from this course will be saved.
     * @throws UnsupportedEncodingException
     * @throws DocumentException
     * @throws IOException
     */
    public void saveLectures(Course course) throws UnsupportedEncodingException, DocumentException, IOException {
        for (Iterator<Lecture> i = course.getLectures().getLectures()
        .iterator(); i.hasNext();) {
            saveLecture(i.next());
        }
    }
    
    /**
     * Loads a lecture of the course
     * @param course Lectures from this course will be loaded.
     * @param id Lecture ID
     * @return Lecture
     * @throws MalformedURLException
     * @throws DocumentException
     */
    public Lecture loadLecture(Course course, String id)
    throws MalformedURLException, DocumentException {
        Lectures lectures = course.getLectures();
        Lecture lecture = new Lecture(lectures, id);
        
        SAXReader xmlReader = new SAXReader();
        Document doc = xmlReader.read(lecture.getFile());
        Description desc = lecture.getDescription();
        desc.setTitle(doc.getRootElement().elementText("title"));
        desc.setBody(doc.getRootElement().elementText("body"));
        // description changed, updating...
        lecture.updateNameInTree();
        
        lectures.addLecture(lecture);
        return lecture;
    }
    
    /**
     * Builds up a new XML document for lecture using DOM4J factory
     * and saves it.
     * @param lecture This lecture will be saved.
     * @throws UnsupportedEncodingException
     * @throws DocumentException
     * @throws IOException
     */
    public void saveLecture(Lecture lecture)
    throws UnsupportedEncodingException, DocumentException, IOException {
        Document doc = factory.createDocument();
        Element root = factory.createElement("lecture");
        root.addAttribute("id", lecture.getId());
        doc.add(root);
        Element title = factory.createElement("title");
        title.setText(lecture.getDescription().getTitle());
        Element body = factory.createElement("body");
        body.setText(lecture.getDescription().getBody());
        root.add(title);
        root.add(body);
        write(doc, lecture.getFile());
    }
    
    /**
     * Loads a lectures' info file
     * @param course Lectures from this course will be used to load info file.
     * @return lectures
     * @throws MalformedURLException
     * @throws DocumentException
     */
    public Lectures loadLecturesInfo(Course course)
    throws MalformedURLException, DocumentException {
        Lectures lectures = course.getLectures();
        
        SAXReader xmlReader = new SAXReader();
        Document doc = xmlReader.read(lectures.getInfoFile());
        Description desc = lectures.getDescription();
        desc.setTitle(doc.getRootElement().elementText("title"));
        desc.setBody(doc.getRootElement().elementText("body"));
        
        return lectures;
    }
    
    /**
     * Builds up a new XML document for lectures info-file using DOM4J factory
     * and saves it.
     * @param lectures This lectures' inf-file will be saved.
     * @throws UnsupportedEncodingException
     * @throws DocumentException
     * @throws IOException
     */
    public void saveLecturesInfo(Lectures lectures)
    throws UnsupportedEncodingException, DocumentException, IOException {
        Document doc = factory.createDocument();
        Element root = factory.createElement("section");
        doc.add(root);
        Element title = factory.createElement("title");
        title.setText(lectures.getDescription().getTitle());
        Element body = factory.createElement("body");
        body.setText(lectures.getDescription().getBody());
        root.add(title);
        root.add(body);
        write(doc, lectures.getInfoFile());
    }
    
    /**
     * Builds up a new XML document for lectures file (includes) using DOM4J 
     * factory and saves it.
     * @param lectures This lectures' file will be saved.
     * @throws java.io.UnsupportedEncodingException 
     * @throws org.dom4j.DocumentException 
     * @throws java.io.IOException 
     */
    public void saveLecturesInclude(Lectures lectures)
    throws UnsupportedEncodingException, DocumentException, IOException {
        Namespace ns = new Namespace("xi", "http://tomp.sf.net/ns/axi/1.0");
        Document doc = factory.createDocument();
        Element root = factory.createElement("article");
        root.addAttribute("lang","cs");
        root.add(ns);
        doc.add(root);
        Element title = factory.createElement("title");
        title.addText(lectures.getDescription().getTitle());
        root.add(title);
        
        Element include = factory.createElement("xi:include");
        include.addAttribute("href","lectures-info.xml");
        root.add(include);
        Element include2 = factory.createElement("xi:include");
        include2.addAttribute("href","navigation.xml");
        root.add(include2);
        
        for (Lecture l: lectures.getLectures()) {
            Element include3 = factory.createElement("xi:include");
            include3.addAttribute("href","lectures/" + l.getId() + ".xml");
            root.add(include3);
        }
        
        write(doc, lectures.getFile());
    }
    
    /**
     * Loads a tasks' info file
     * @param course Tasks from this course will be used to load info file.
     * @return tasks
     * @throws MalformedURLException
     * @throws DocumentException
     */
    public Tasks loadTasksInfo(Course course)
    throws MalformedURLException, DocumentException {
        Tasks tasks = course.getTasks();
        
        SAXReader xmlReader = new SAXReader();
        Document doc = xmlReader.read(tasks.getInfoFile());
        Description desc = tasks.getDescription();
        desc.setTitle(doc.getRootElement().elementText("title"));
        desc.setBody(doc.getRootElement().elementText("body"));
        
        return tasks;
    }
    
    /**
     * Builds up a new XML document for tasks info-file using DOM4J factory
     * and saves it.
     * @param tasks This tasks' info-file will be saved.
     * @throws UnsupportedEncodingException
     * @throws DocumentException
     * @throws IOException
     */
    public void saveTasksInfo(Tasks tasks)
    throws UnsupportedEncodingException, DocumentException, IOException {
        Document doc = factory.createDocument();
        Element root = factory.createElement("section");
        doc.add(root);
        Element title = factory.createElement("title");
        title.setText(tasks.getDescription().getTitle());
        Element body = factory.createElement("body");
        body.setText(tasks.getDescription().getBody());
        root.add(title);
        root.add(body);
        write(doc, tasks.getInfoFile());
    }
    
    /**
     * Builds up a new XML document for tasks file (includes) using DOM4J 
     * factory and saves it.
     * @param tasks This tasks' file will be saved.
     * @throws java.io.UnsupportedEncodingException 
     * @throws org.dom4j.DocumentException 
     * @throws java.io.IOException 
     */
    public void saveTasksInclude(Tasks tasks)
    throws UnsupportedEncodingException, DocumentException, IOException {
        Namespace ns = new Namespace("xi", "http://tomp.sf.net/ns/axi/1.0");
        Document doc = factory.createDocument();
        Element root = factory.createElement("article");
        root.addAttribute("lang","cs");
        root.add(ns);
        doc.add(root);
        Element title = factory.createElement("title");
        title.addText(tasks.getDescription().getTitle());
        root.add(title);
        
        Element include = factory.createElement("xi:include");
        include.addAttribute("href","tasks-info.xml");
        root.add(include);
        Element include2 = factory.createElement("xi:include");
        include2.addAttribute("href","navigation.xml");
        root.add(include2);
        
        for (Task t: tasks.getTasks()) {
            Element include3 = factory.createElement("xi:include");
            include3.addAttribute("href","tasks/" + t.getId() + ".xml");
            root.add(include3);
        }
        
        write(doc, tasks.getFile());
    }    
    
    /**
     * Loads a practices' info file
     * @param course Practices from this course will be used to load info file.
     * @return practices
     * @throws MalformedURLException
     * @throws DocumentException
     */
    public Practices loadPracticesInfo(Course course)
    throws MalformedURLException, DocumentException {
        Practices practices = course.getPractices();
        
        SAXReader xmlReader = new SAXReader();
        Document doc = xmlReader.read(practices.getInfoFile());
        Description desc = practices.getDescription();
        desc.setTitle(doc.getRootElement().elementText("title"));
        desc.setBody(doc.getRootElement().elementText("body"));
        
        return practices;
    }
    
    /**
     * Builds up a new XML document for practices info file using DOM4J factory
     * and saves it.
     * @param practices This practices' info file will be saved.
     * @throws UnsupportedEncodingException
     * @throws DocumentException
     * @throws IOException
     */
    public void savePracticesInfo(Practices practices)
    throws UnsupportedEncodingException, DocumentException, IOException {
        Document doc = factory.createDocument();
        Element root = factory.createElement("section");
        doc.add(root);
        Element title = factory.createElement("title");
        title.setText(practices.getDescription().getTitle());
        Element body = factory.createElement("body");
        body.setText(practices.getDescription().getBody());
        root.add(title);
        root.add(body);
        write(doc, practices.getInfoFile());
    }
    
    /**
     * Builds up a new XML document for practices file (includes) using DOM4J 
     * factory and saves it.
     * @param practices This practices' file will be saved.
     * @throws java.io.UnsupportedEncodingException 
     * @throws org.dom4j.DocumentException 
     * @throws java.io.IOException 
     */
    public void savePracticesInclude(Practices practices)
    throws UnsupportedEncodingException, DocumentException, IOException {
        Namespace ns = new Namespace("xi", "http://tomp.sf.net/ns/axi/1.0");
        Document doc = factory.createDocument();
        Element root = factory.createElement("article");
        root.addAttribute("lang","cs");
        root.add(ns);
        doc.add(root);
        Element title = factory.createElement("title");
        title.addText(practices.getDescription().getTitle());
        root.add(title);
        
        Element include = factory.createElement("xi:include");
        include.addAttribute("href","practices-info.xml");
        root.add(include);
        Element include2 = factory.createElement("xi:include");
        include2.addAttribute("href","navigation.xml");
        root.add(include2);
        
        for (Practice p: practices.getPractices()) {
            Element include3 = factory.createElement("xi:include");
            include3.addAttribute("href","practices/" + p.getId() + ".xml");
            root.add(include3);
        }
        
        write(doc, practices.getFile());
    }       
    
    /**
     * Loads a projects' info file
     * @param course Projects from this course will be used to load info file.
     * @return projects
     * @throws MalformedURLException
     * @throws DocumentException
     */
    public Projects loadProjectsInfo(Course course)
    throws MalformedURLException, DocumentException {
        Projects projects = course.getProjects();
        
        SAXReader xmlReader = new SAXReader();
        Document doc = xmlReader.read(projects.getInfoFile());
        Description desc = projects.getDescription();
        desc.setTitle(doc.getRootElement().elementText("title"));
        desc.setBody(doc.getRootElement().elementText("body"));
        
        return projects;
    }
    
    /**
     * Builds up a new XML document for projects info file using DOM4J factory
     * and saves it.
     * @param projects This projects' info file will be saved.
     * @throws UnsupportedEncodingException
     * @throws DocumentException
     * @throws IOException
     */
    public void saveProjectsInfo(Projects projects)
    throws UnsupportedEncodingException, DocumentException, IOException {
        Document doc = factory.createDocument();
        Element root = factory.createElement("section");
        doc.add(root);
        Element title = factory.createElement("title");
        title.setText(projects.getDescription().getTitle());
        Element body = factory.createElement("body");
        body.setText(projects.getDescription().getBody());
        root.add(title);
        root.add(body);
        write(doc, projects.getInfoFile());
    }
    
    /**
     * Builds up a new XML document for projects file (includes) using DOM4J 
     * factory and saves it.
     * @param projects This projects' file will be saved.
     * @throws java.io.UnsupportedEncodingException 
     * @throws org.dom4j.DocumentException 
     * @throws java.io.IOException 
     */
    public void saveProjectsInclude(Projects projects)
    throws UnsupportedEncodingException, DocumentException, IOException {
        Namespace ns = new Namespace("xi", "http://tomp.sf.net/ns/axi/1.0");
        Document doc = factory.createDocument();
        Element root = factory.createElement("article");
        root.addAttribute("lang","cs");
        root.add(ns);
        doc.add(root);
        Element title = factory.createElement("title");
        title.addText(projects.getDescription().getTitle());
        root.add(title);
        
        Element include = factory.createElement("xi:include");
        include.addAttribute("href","projects-info.xml");
        root.add(include);
        Element include2 = factory.createElement("xi:include");
        include2.addAttribute("href","navigation.xml");
        root.add(include2);
        
        for (Project p: projects.getProjects()) {
            Element include3 = factory.createElement("xi:include");
            include3.addAttribute("href","projects/" + p.getId() + ".xml");
            root.add(include3);
        }
        
        write(doc, projects.getFile());
    }      
    
    /**
     * Loads a task of the course
     * @param course Tasks from this course will be loaded.
     * @param id Task ID
     * @return Task
     * @throws MalformedURLException
     * @throws DocumentException
     */
    public Task loadTask(Course course, String id)
    throws MalformedURLException, DocumentException {
        Tasks tasks = course.getTasks();
        Task task = new Task(tasks, id);
        
        SAXReader xmlReader = new SAXReader();
        Document doc = xmlReader.read(task.getFile());
        Description desc = task.getDescription();
        desc.setTitle(doc.getRootElement().elementText("title"));
        desc.setBody(doc.getRootElement().elementText("body"));
        // description changed, updating...
        task.updateNameInTree();
        
        tasks.addTask(task);
        return task;
    }
    
    /**
     * Loads tasks for a course.
     * @param course Tasks from this course will be loaded.
     * @throws MalformedURLException
     * @throws DocumentException
     */
    public void loadTasks(Course course) throws MalformedURLException,
            DocumentException {
        String[] taskIds = listTaskIds(course.getTasks());
        for (int i = 0; i < taskIds.length; i++) {
            loadTask(course, taskIds[i]);
        }
    }
    
    /**
     * Builds up a new XML document for task using DOM4J factory
     * and saves it.
     * @param t This task will be saved.
     * @throws UnsupportedEncodingException
     * @throws DocumentException
     * @throws IOException
     */
    public void saveTask(Task t) throws UnsupportedEncodingException, DocumentException, IOException {
        Document doc = factory.createDocument();
        Element root = factory.createElement("task");
        root.addAttribute("id", t.getId());
        doc.add(root);
        Element title = factory.createElement("title");
        title.setText(t.getDescription().getTitle());
        Element body = factory.createElement("body");
        body.setText(t.getDescription().getBody());
        root.add(title);
        root.add(body);
        write(doc, t.getFile());
    }
    
    /**
     * Loads a practice of the course
     * @param course Practices from this course will be loaded.
     * @param id Practice ID
     * @return Practice
     * @throws MalformedURLException
     * @throws DocumentException
     */
    public Practice loadPractice(Course course, String id)
    throws MalformedURLException, DocumentException {
        Practices practices = course.getPractices();
        Practice practice = new Practice(practices, id);
        
        SAXReader xmlReader = new SAXReader();
        Document doc = xmlReader.read(practice.getFile());
        Description desc = practice.getDescription();
        desc.setTitle(doc.getRootElement().elementText("title"));
        desc.setBody(doc.getRootElement().elementText("body"));
        // description changed, updating...
        practice.updateNameInTree();
        
        practices.addPractice(practice);
        return practice;
    }
    
    /**
     * Loads practices for a course.
     * @param course Practices from this course will be loaded.
     * @throws MalformedURLException
     * @throws DocumentException
     */
    public void loadPractices(Course course) throws MalformedURLException,
            DocumentException {
        String[] practiceIds = listPracticeIds(course.getPractices());
        for (int i = 0; i < practiceIds.length; i++) {
            loadPractice(course, practiceIds[i]);
        }
    }
    
    /**
     * Builds up a new XML document for practice using DOM4J factory
     * and saves it.
     * @param t This practice will be saved.
     * @throws UnsupportedEncodingException
     * @throws DocumentException
     * @throws IOException
     */
    public void savePractice(Practice t) throws UnsupportedEncodingException, DocumentException, IOException {
        Document doc = factory.createDocument();
        Element root = factory.createElement("practice");
        root.addAttribute("id", t.getId());
        doc.add(root);
        Element title = factory.createElement("title");
        title.setText(t.getDescription().getTitle());
        Element body = factory.createElement("body");
        body.setText(t.getDescription().getBody());
        root.add(title);
        root.add(body);
        write(doc, t.getFile());
    }
    
    /**
     * Loads a project of the course
     * @param course Projects from this course will be loaded.
     * @param id Project ID
     * @return Project
     * @throws MalformedURLException
     * @throws DocumentException
     */
    public Project loadProject(Course course, String id)
    throws MalformedURLException, DocumentException {
        Projects projects = course.getProjects();
        Project project = new Project(projects, id);
        
        SAXReader xmlReader = new SAXReader();
        Document doc = xmlReader.read(project.getFile());
        Description desc = project.getDescription();
        desc.setTitle(doc.getRootElement().elementText("title"));
        desc.setBody(doc.getRootElement().elementText("body"));
        // description changed, updating...
        project.updateNameInTree();        
        
        projects.addProject(project);
        return project;
    }
    
    /**
     * Loads projects for a course.
     * @param course Projects from this course will be loaded.
     * @throws MalformedURLException
     * @throws DocumentException
     */
    public void loadProjects(Course course) throws MalformedURLException,
            DocumentException {
        String[] projectsIds = listProjectIds(course.getProjects());
        for (int i = 0; i < projectsIds.length; i++) {
            loadProject(course, projectsIds[i]);
        }
    }
    
    /**
     * Builds up a new XML document for project using DOM4J factory
     * and saves it.
     * @param t This project will be saved.
     * @throws UnsupportedEncodingException
     * @throws DocumentException
     * @throws IOException
     */
    public void saveProject(Project t) throws UnsupportedEncodingException, DocumentException, IOException {
        Document doc = factory.createDocument();
        Element root = factory.createElement("project");
        root.addAttribute("id", t.getId());
        doc.add(root);
        Element title = factory.createElement("title");
        title.setText(t.getDescription().getTitle());
        Element body = factory.createElement("body");
        body.setText(t.getDescription().getBody());
        root.add(title);
        root.add(body);
        write(doc, t.getFile());
    }
    
    /**
     * Browses the tree's directory and gets the IDs (= names of the courses'
     * directories) of the courses.
     * @param tree This tree (courses' directories) will be browsed.
     * @return IDs of the courses
     */
    protected String[] listCourseIds(CourseTree tree) {
        File treeDir = tree.getDirectory();
        File[] courseDirs = treeDir.listFiles(new FileFilter() {
            public boolean accept(File f) {
                return f.isDirectory()
                && !f.getName().equalsIgnoreCase("system")
                && !f.getName().startsWith(".");
            }
        });
        String[] ids = new String[courseDirs.length];
        for (int i = 0; i < courseDirs.length; i++) {
            ids[i] = courseDirs[i].getName();
        }
        return ids;
    }
    
    /**
     * Browses the lectures' directories and gets the IDs of the lectures
     * (= names of the lectures' files).
     * @param lectures These lectures will be browsed.
     * @return IDs of the lectures
     */
    protected String[] listLectureIds(Lectures lectures) {
        File lecturesDir = lectures.getDirectory();
        File[] lectureFiles = lecturesDir.listFiles(new FileFilter() {
            public boolean accept(File f) {
                return true;
            }
        });
        if (lectureFiles == null) {
            lectureFiles = new File[0];
        }
        String[] ids = new String[lectureFiles.length];
        for (int i = 0; i < lectureFiles.length; i++) {
            String name = lectureFiles[i].getName();
            ids[i] = name.substring(0, name.length() - 4);
        }
        return ids;
    }
    
    /**
     * Browses the tasks' directories and gets the IDs of the tasks
     * (= names of the tasks' files).
     * @param tasks These tasks will be browsed.
     * @return IDs of the tasks
     */
    protected String[] listTaskIds(Tasks tasks) {
        File tasksDir = tasks.getDirectory();
        File[] taskFiles = tasksDir.listFiles(new FileFilter() {
            public boolean accept(File f) {
                return true;
            }
        });
        if (taskFiles == null) {
            taskFiles = new File[0];
        }
        String[] ids = new String[taskFiles.length];
        for (int i = 0; i < taskFiles.length; i++) {
            String name = taskFiles[i].getName();
            ids[i] = name.substring(0, name.length() - 4);
        }
        return ids;
    }
    
    /**
     * Browses the practices' directories and gets the IDs of the practices
     * (= names of the practices' files).
     * @param practices These practices will be browsed.
     * @return IDs of the practices
     */
    protected String[] listPracticeIds(Practices practices) {
        File practicesDir = practices.getDirectory();
        File[] practiceFiles = practicesDir.listFiles(new FileFilter() {
            public boolean accept(File f) {
                return true;
            }
        });
        if (practiceFiles == null) {
            practiceFiles = new File[0];
        }
        String[] ids = new String[practiceFiles.length];
        for (int i = 0; i < practiceFiles.length; i++) {
            String name = practiceFiles[i].getName();
            ids[i] = name.substring(0, name.length() - 4);
        }
        return ids;
    }
    
    /**
     * Browses the projects' directories and gets the IDs of the projects
     * (= names of the projects' files).
     * @param projects These projects will be browsed.
     * @return IDs of the projects
     */
    protected String[] listProjectIds(Projects projects) {
        File projectsDir = projects.getDirectory();
        File[] projectFiles = projectsDir.listFiles(new FileFilter() {
            public boolean accept(File f) {
                return true;
            }
        });
        if (projectFiles == null) {
            projectFiles = new File[0];
        }
        String[] ids = new String[projectFiles.length];
        for (int i = 0; i < projectFiles.length; i++) {
            String name = projectFiles[i].getName();
            ids[i] = name.substring(0, name.length() - 4);
        }
        return ids;
    }
    
    /**
     * Writes the XML document (e. g. of a lecture) to a file.
     * @param doc The generated XML document which will
     * be saved.
     * @param f The names of the file to save do XML document to.
     * @throws DocumentException
     * @throws UnsupportedEncodingException
     * @throws IOException
     */
    public void write(Document doc, File f) throws DocumentException,
            UnsupportedEncodingException, IOException {
        OutputFormat outformat = OutputFormat.createPrettyPrint();
        //		   outformat.setEncoding(aEncodingScheme);
        XMLWriter writer = new XMLWriter(new FileOutputStream(f), outformat);
        writer.write(doc);
        writer.flush();
    }
    
}